##################################################
#	Bent Nielsen
#	16 April 2020
#
#	Now-casting COVID-19 fatalities: track by death-date (not by report-date)
#	SM Bird and B Nielsen
#
#	Uses data NHS data for England from
#	https://www.england.nhs.uk/statistics/statistical-work-areas/covid-19-daily-deaths/
#	along with some older data.
#
#	Uses package apc
#

##################################################
#	Parameters, user defined
##################################################
#	selection of data and number of now-casts
n.rep	<- 7	# how report periods to use in each analysis? 7 would be the last week of reports.
n.coh	<- 10	# how many cohorts/date-of-deaths to use in analysis? 10 would get most of the tail. Requires n.coh >= n.rep.
n.now	<- 1	# number of nowcasts? Choose several for recursive evaluation. 1 or 5 good choices
n.last	<- 0	# number of recent periods to drop. 0 would use most recent data
n.coh.x	<- 14	# how many cohorts/date-of-deaths on x axis in plot
#	selection of quantiles for error bands in distribution nowcasting
quantiles 	<- c(0.025,0.975)

##################################################
#	AC now-cast
##################################################
#	title in plot
main <-	paste(region,": Nowcasts by rolling reporting date","\n","using ",n.rep," reports & ",n.coh," dates-of-deaths",sep="")

#	set parameter values
nrow <- nrow(m.total.sub)-n.last	# most recent reporting dates to use
n.now.trunc <- min(n.now,nrow)		# truncate the number of nowcasts, by what is feasible given data
coh_per.dif <- n.coh - n.rep		# how many cohorts more than periods?
if(coh_per.dif<0) print("WARNING: n.coh - n.rep")

##########
#	plot parameters
ymax	<- ceiling(max(m.total.sub,na.rm=TRUE)*1.1/100)
ylim	<- c(0,ymax*100)
shift 	<- 0.1
x		<- ncol(data.sub$response)+1-n.last-seq(1:n.coh.x)

##################################
#	plot reported data
##################################
#	save to file?

if(l.save.plot.as.svg)
	svg(filename=paste(drive.plot,"FigNowcast",region,date,"_",n.rep,"reports",n.coh,"cohorts.svg",sep=""),width=5, height=5, pointsize=12)
if(l.save.plot.as.svg && n.now>1)
	svg(filename=paste(drive.plot,"FigNowcastRecursive",region,Sys.Date(),"_",n.rep,"reports",n.coh,"cohorts.svg",sep=""),width=5, height=5, pointsize=12)

#	plot most recent data
plot(x,m.total.sub[nrow,x],pch=4,ylim=ylim,xlab="",ylab="deaths",xaxt="n",main=main,las=2)
#   plot older data
if(n.now.trunc>1)
	for(i in 2:n.now.trunc)
		points(x-(i-1)*shift,m.total.sub[nrow+1-i,x],pch=4,col=i)

#	add grey horizontal lines
for(j in 0:ymax)
	lines(c(min(x)-0.04*length(x),max(x)+0.04*length(x)),j*100*c(1,1),col=grey(0.9))

#	add legend
legend("bottomleft",rownames(m.total.sub)[nrow+1-1:n.now.trunc],lty=1,col=1:n.now.trunc,title="reporting dates")
#	add x axis, with labels from data matrix
axis(1,at=x,labels=colnames(m.total.sub)[x],las=2)
mtext("date-of-death",side=1,line=4)

##################################
#	plot nowcasts with error bands
##################################
for(i in n.now.trunc:1)	# choose reporting date
{	#	select a sub-sample
	per.cut.upper	<- i-1+n.last
	per.cut.lower	<- ncol(data.sub$response) - n.rep - per.cut.upper - data.sub$per.zero
	coh.cut.lower	<- ncol(data.sub$response) - n.coh - per.cut.upper
	data.ac <- apc.data.list.subset(data.sub,0,0,per.cut.lower,per.cut.upper,coh.cut.lower,0,suppress.warning=suppress.warning)
	#	set indices for considered cohorts
	i.ac	<- coh.cut.lower+2:(n.rep+coh_per.dif)
	#	find totals for reporting date
	v.total <- m.total.sub[nrow+1-i,i.ac]
	#	fit an AC=age-cohort model
	fit.ac	<- apc.fit.model(data.ac,"od.poisson.response","AC")
	#	nowcast/forecast lower triangle
	now.ac	<- apc.forecast.ac(fit.ac,quantiles=quantiles)
	#	find sums by cohort (column) in lower triangle
	now.ac$response.forecast.coh
	#	add observed totals
	total.now.ac <- now.ac$response.forecast.coh
	total.now.ac[,c(1,6,7)]	<- total.now.ac[,c(1,6,7)] + v.total
	#	find x values
	x.sel	<- i.ac-(i-1)*shift
	#	print nowcasts
	#		first points
	points(x.sel,total.now.ac[,1],col=i,pch=3,cex=0.8)
	#		then error bands
	for(j in 1:length(i.ac))
	{	x0 	<- 	x1	<- x.sel[j]
		y0	<-	total.now.ac[j,6]
		y1	<-	total.now.ac[j,7]
		if(y1-y0>1)	# error message if band is too small
			arrows(x0,y0,x1,y1,	code=3, angle=90, length=0.04, col=i)
	}		
	#	print data used and nowcasts for most recent reporting date
	if(i==1)
	{	print("data used for most recent nowcast")
		print(data.ac$response)
		print("most recent nowcast")
		print(region)
		print(total.now.ac[,c(1,6,7)])
	}	
}

#	add lines to most recent reporting data & nowcast
x.data	<- x[(nrow(total.now.ac)+1):length(x)]
y.data	<- m.total.sub[nrow,ncol(data.sub$response)-n.last-n.coh.x+(length(x)-nrow(total.now.ac)):1]
lines(x.data,y.data,lty=1)
x.now	<- x[(nrow(total.now.ac)+1):1]
y.now	<- total.now.ac[,1]
lines(x.now,c(y.data[1],y.now),lty=2)

if(l.save.plot.as.svg)
	dev.off()

